// src/backend/simulationStatus.cc - Copyright 2005, 2006, University
//               of Padova, dept. of Pure and Applied
//               Mathematics
//
// This file is part of SGPEMv2.
//
// This is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// SGPEMv2 is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with SGPEMv2; if not, write to the Free Software
// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#include "simulation_status.hh"
using namespace sgpem;
using namespace std;
using namespace memory;

/** \brief Creates the SimulationStatus object with no SchedulableStatus object
	
 */
SimulationStatus::SimulationStatus()
{
}

SimulationStatus::SimulationStatus(const SimulationStatus& dx)
  :_set(dx._set)
{	
}



/** \brief Adds a SchedulableStatus which represents a running Schedulable object.
	
    Adds a SchedulableStatus which represents a running Schedulable object.
    Note that a copy of SchedulableStatus will be created	
*/
void
SimulationStatus::set_running(SchedulableStatus entity)
{
  if (entity.get_state() != SchedulableStatus::state_running)
    entity.set_state(SchedulableStatus::state_running);
  _set.push_back(entity);
}

/** \brief Looks for a running Schedulable object and returns ist state.
    Looks for a running Schedulable object and returns its state. There can be only 
    one running schedulable object. If no running schedulable object is found will be returned
    the NULL pointer.
*/
					   
smart_ptr<SchedulableStatus>
SimulationStatus::get_running()
{
  for (list<SchedulableStatus>::iterator f=_set.begin(); f != _set.end(); f++)
    if (f->get_state() == SchedulableStatus::state_running){	//there can be only ONE running schedulable objext
      return smart_ptr<SchedulableStatus>(new SchedulableStatus(*f));
    }
  return smart_ptr<SchedulableStatus>(NULL);
}

smart_ptr<const SchedulableStatus>
SimulationStatus::get_running() const
{
  for (list<SchedulableStatus>::const_iterator f=_set.begin(); f != _set.end(); f++)
    if (f->get_state() == SchedulableStatus::state_running){	//there can be only ONE running schedulable objext
      return smart_ptr<const SchedulableStatus>(new SchedulableStatus(*f));
    }
  return smart_ptr<const SchedulableStatus>(NULL);	//NOT FOUND RUNNING ENTITIES
}


/** 
    \brief Returns TRUE if the two objects have the same SchedulableStatus objects
*/
bool 								
SimulationStatus::operator==(const SimulationStatus& dx) const
{
  if (_set.size() != dx._set.size())
    return false;
	
  //check if dx has ALL and ONLY the elements holded by _set with no order importance
  for(list<SchedulableStatus>::const_iterator f=_set.begin(); f != _set.end(); f++)
    if (find(dx._set.begin(), dx._set.end(), *f) == dx._set.end())		//element NOT found!!
      return false;
	
  return true;
}
